#ifndef CONFIG_H
#define CONFIG_H

#include <QObject>

class QSettings;

class Config : public QObject
{
    Q_OBJECT
public:
    explicit Config(QObject *parent = nullptr);

    enum DesktopMode {
        Tablet,
        EduScreen,
    };
    Q_ENUM(DesktopMode);

    struct ContainerInfo
    {
        QString containerName;
        QStringList desktopFiles;
        QList<int> placeholder;
        bool titleEditable;
        bool containerEditable;
        QString containerCategory;
    };

    DesktopMode desktopMode() const;

    QStringList appWidgetList() const;

    ///
    /// \brief desktopLayout
    /// \code
    ///     int row, column;
    ///     auto [row, column] = desktopLayout();
    /// \endcode
    /// \return
    ///
    std::tuple<int, int> desktopLayout() const;

    QMap<QString, QStringList> defaultGroups() const;
    QList<ContainerInfo> defaultContainers() const;
    QList<int> layoutList(QString layout) const;

private:
    QSettings *m_confSettings;
    const QString kConfFile = "/etc/ukui-tablet-desktop.conf";
};

#endif // CONFIG_H
