#include "taskbarappmanager.h"
#include <QStandardPaths>
#include <QFile>
#include <QSettings>
#include "ukuisettings.h"

static const QStringList defaultTaskbarApps = {
    "/usr/share/applications/peony.desktop",
    "/usr/share/applications/kylin-software-center.desktop",
    "/usr/share/applications/wps-office-wps.desktop"
};

TaskbarAppManager::TaskbarAppManager(QObject *parent)
    : QObject{parent}
{
    loadSettingsFile();

    m_fileWatcher = new QFileSystemWatcher(this);
    if (m_settingsFile.isEmpty()) {
        qWarning() << "m_settingsFile is empty !!!" << __func__;
        return;
    }
    m_fileWatcher->addPath(m_settingsFile);
    connect(m_fileWatcher, SIGNAL(fileChanged(const QString &)), this, SLOT(taskBarDataChanged()));
}

QStringList TaskbarAppManager::appDesktopfiles() const
{
    return m_appsDesktopFiles;
}

void TaskbarAppManager::checkAppDesktopfiles(const QStringList &preDesktopfiles)
{
    m_appsDesktopFiles = preDesktopfiles;
    taskBarDataChanged();
}

void TaskbarAppManager::updatePanelConf(const QList<QString> &taskbarData)
{
    QList<QMap<QString, QVariant>> hashList;
    for (int i = 0; i < taskbarData.size(); ++i) {
        QString desktopName = taskbarData.at(i);
        QHash<QString, QString> m_settingsMap;
        m_settingsMap["desktop"] = desktopName;

        QMap<QString, QVariant> map;
        QHashIterator<QString, QString> it(m_settingsMap);
        while (it.hasNext()) {
            it.next();
            map[it.key()] = it.value();
        }
        hashList << map;
    }
    Settings settings(m_settingsFile, QSettings::IniFormat,"taskbar", this);
    settings.setArray("apps", hashList);
}

void TaskbarAppManager::loadSettingsFile()
{
    auto locations = QStandardPaths::standardLocations(QStandardPaths::ConfigLocation);
    if (!locations.isEmpty()) {
        m_settingsFile = locations.constFirst() + "/ukui/panel.conf";
        if (!QFile::exists(m_settingsFile)) {
            m_settingsFile.clear();
        }
    } else {
        if (QFile::exists("/usr/share/ukui/panel.conf")) {
            m_settingsFile = "/usr/share/ukui/panel.conf";
        }
    }
    if (!m_settingsFile.isEmpty()) {
        Settings settings(m_settingsFile, QSettings::IniFormat,"taskbar", this);
        auto taskbarApps = settings.readArray("apps");
        for (auto &app : qAsConst(taskbarApps)) {
            m_appsDesktopFiles.append(app.value("desktop").toString());
        }
    } else {
        qWarning() << "settingsFile is empty !!!" << __func__;
        m_appsDesktopFiles = defaultTaskbarApps;
    }
}

QStringList TaskbarAppManager::readTaskbarPanelConf()
{
    QStringList taskBarAppDesktop;
    if (m_settingsFile.isEmpty()) {
        qWarning() << "m_settingsFile is empty !!!" << __func__;
        return taskBarAppDesktop;
    }
    Settings settings(m_settingsFile, QSettings::IniFormat,"taskbar", this);
    QList<QMap<QString, QVariant> > taskBarApp = settings.readArray("apps");
    for (int var = 0; var < taskBarApp.size(); ++var) {
        taskBarAppDesktop.push_back(taskBarApp[var]["desktop"].toString());
    }
    return taskBarAppDesktop;
}

void TaskbarAppManager::taskBarDataChanged()
{
    qDebug() << "任务栏配置文件发生改变！" << __FUNCTION__;
    if (m_settingsFile.isEmpty()) {
        qWarning() << "m_settingsFile is empty !!!" << __func__;
        return;
    }
    m_fileWatcher->addPath(m_settingsFile);

    QStringList taskBarAppDesktop;
    taskBarAppDesktop = readTaskbarPanelConf();

    if (taskBarAppDesktop == m_appsDesktopFiles) {
        return;
    }
    if (taskBarAppDesktop.count() > m_appsDesktopFiles.count()) {
        Q_EMIT taskbarAppChanged(taskBarAppDesktop, TaskbarAction::Add);
    } else if (taskBarAppDesktop.count() < m_appsDesktopFiles.count()) {
        Q_EMIT taskbarAppChanged(taskBarAppDesktop, TaskbarAction::Delete);
    } else {
        Q_EMIT taskbarAppChanged(taskBarAppDesktop, TaskbarAction::Move);
    }
    m_appsDesktopFiles = taskBarAppDesktop;
}

