#ifndef TRAYICONWATCHER_H
#define TRAYICONWATCHER_H

#include <QObject>
#include <QGSettings>
#include <QDebug>

#if QT_VERSION >= QT_VERSION_CHECK(5,15,0)
#include <QtQml/qqml.h>
#endif

class TrayIconWatcher : public QObject
{
    Q_OBJECT
#if QT_VERSION >= QT_VERSION_CHECK(5,15,0)
    QML_ELEMENT
#endif
    Q_PROPERTY(QString iconSource READ iconSource WRITE setIconSource NOTIFY iconSourceChanged)
    Q_PROPERTY(int type READ type WRITE setType NOTIFY typeChanged)
    Q_PROPERTY(QString timeformat READ timeformat WRITE settimeformat NOTIFY timeformatChanged)
    Q_PROPERTY(QString dateformat READ dateformat WRITE setdateformat NOTIFY dateformatChanged)

public:
    enum IconType {
        Volume,
        Power,
        Message,
        Network,
        Bluetooth,
        TimeDate,
    };
    Q_ENUMS(IconType)

    explicit TrayIconWatcher(QObject *parent = nullptr);

    QString iconSource();
    void setIconSource(const QString &source);

    int type();
    void setType(const int &type);

    QString timeformat();
    void settimeformat(const QString &timeformat);

    QString dateformat();
    void setdateformat(const QString &dateformat);

    void getVolume();
    void initPowerDbusaddrest();
    void getTimeDateState();

    void setVolumeIcon(int value);
    void setPowerIcon(int powerStateValue, int powerValue);
    void setMessageIcon(bool value);
    void setNetIcon(int value, bool state);
    void setBluetoothIcon(bool state);

private:
    QString m_iconSource;
    int m_type;
    QString m_timeformat;
    QString m_dateformat;
    QGSettings *timeSetting;
    QString powerpath;
    int netconnectType = 0; //wire = 1, wifi = 2


signals:
    void iconSourceChanged();
    void typeChanged();
    void timeformatChanged();
    void dateformatChanged();

public slots:
    void initPowerValue();
    void getWifiValue();
    void getMessageState();
    void getBluetoothSwitch();
    void setVolume(int sound);
    void setMuteState(bool state);
};

#endif // TRAYICONWATCHER_H
