#include "appiconimageprovider.h"
#include <QIcon>
#include <QGSettings>
#include <QMutex>
#include <QMutexLocker>
#include <KIconTheme>
#include <QDebug>

extern QMutex qiconMutex;

AppIconImageProvider::AppIconImageProvider()
    : QQuickImageProvider(QQuickImageProvider::Pixmap)
{

}

AppIconImageProvider::~AppIconImageProvider()
{
}

QPixmap AppIconImageProvider::requestPixmap(const QString &id, QSize *size, const QSize &requestedSize)
{
    Q_UNUSED(size)
    if (id.startsWith("file#")) {
        auto items = id.split("#");
        if (items.size() < 2) {
            qWarning() << "AppIconImageProvider: app icon id error, " << id;
            return QPixmap(xDesktopAppIcon());
        }

        return QPixmap(items.at(1));
    }

    if (id.startsWith("theme#")) {
        auto items = id.split("#");
        if (items.size() < 3) {
            qWarning() << "AppIconImageProvider: app icon id error, " << id;
            return QPixmap(xDesktopAppIcon());
        }
        QMutexLocker locker(&qiconMutex);
        QString iconName = items.last();
        QIcon icon = QIcon::fromTheme(iconName);
        return icon.pixmap(requestedSize.width(), requestedSize.height());
    }

    return QPixmap();
}

QString AppIconImageProvider::xDesktopAppIcon()
{
    QGSettings iconStyle("org.ukui.style");
    QString iconThemeName;

    if (iconStyle.keys().contains("iconThemeName")) {
        iconThemeName = iconStyle.get("iconThemeName").toString();
    }
    KIconTheme iconInfo(iconThemeName);
    return iconInfo.iconPathByName("application-x-desktop", 96, KIconLoader::MatchBest);
}
