import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQml 2.12
import QtGraphicalEffects 1.0
import org.ukui.tablet 1.0
import "."

Item {
    id: root
    property int iconState: LauncherItem.Normal
    property int appId: 0
    property int pageIndex: 0

    property bool selected: true;

    property string icon;
    property int iconWidth: 100  * Style.widthScaleFactor;
    property int iconRadius: 10 * Style.widthScaleFactor;

    property int deleteiconWidth: 30 * Style.widthScaleFactor;
    property int selecticonWidth: 30 * Style.widthScaleFactor;

    signal unload(var appid);
    signal checked(var appid);
    signal unchecked(var appid);

    property bool disableDelete : false

    signal deleteIcon()

    state: {
        switch (iconState) {
        case LauncherItem.Normal: return "Normal";
        case LauncherItem.Editable: return "Editable";
        case LauncherItem.Selectable: return "Selectable";
        }
    }

    states: [
        State {
            name: "Normal"
            StateChangeScript {
                script: {
                    if (deleteiconAnimation.running)
                        deleteiconAnimation.stop();
                    if (appiconAnimation.running)
                        appiconAnimation.stop();
                    if (selecticonAnimation.running)
                        selecticonAnimation.stop();
                    deleteicon.scale = 0;
                    selecticon.scale = 0;
                    appicon.rotation = 0;
                    deleteicon.rotation = 0;
                    if(modelData.type === LauncherItem.AddIcon) {
                        image.width = Qt.binding(function() {
                            return (iconWidth/2)
                        })
                        image.height = Qt.binding(function() {
                            return (iconWidth/2)
                        })

                        mask.visible = true
                    }
                }
            }
        },
        State {
            name: "Editable"
            StateChangeScript {
                script: {
                    deleteiconAnimation.start();
                    appiconAnimation.start();
                }
            }
        },
        State {
            name: "Selectable"
            StateChangeScript {
                script: {
                    selecticonAnimation.start(); }
            }
        }
    ]

    width: iconWidth;
    height: iconWidth;

    Rectangle {
        id: mask;
        width: iconWidth;
        height: iconWidth;
        radius: iconRadius;
        visible: false;
    }
    Image {
        id: image;
        source: icon;
        anchors.centerIn: mask;
        width: iconWidth;
        height: iconWidth;
        sourceSize.width: iconWidth;
        sourceSize.height: iconWidth;
        visible: false;
        smooth: true
        mipmap: true
    }
    OpacityMask {
        id: appicon;
        anchors.fill: image;
        source: image;
        maskSource: mask;
    }

    Image {
        id: deleteicon;
        visible: !disableDelete
        anchors {
            left: appicon.right; leftMargin: (-deleteiconWidth/2);
            top: appicon.top; topMargin: -deleteiconWidth/5;
        }
        source: deleteMouseArea.pressed ? "qrc:/ui/images/icon_delete_click.svg" :
                                    "qrc:/ui/images/icon_delete_default.svg"
        MouseArea {
            id: deleteMouseArea
            anchors.fill: parent
            onClicked: deleteIcon()
        }
    }
    ScaleAnimator {
        id: deleteiconAnimation;
        target: deleteicon;
        from: 0;
        to: 1;
        duration: 500;
    }
    SequentialAnimation {
        id: appiconAnimation;
        loops: AnimatedSprite.Infinite;
        ParallelAnimation {
            RotationAnimation {
                target: appicon;
                from: 0;
                to: 2;
                duration: 150;
            }
            RotationAnimation {
                target: deleteicon;
                from: 0;
                to: 5;
                duration: 120;
            }
        }
        ParallelAnimation {
            RotationAnimation {
                target: appicon;
                from: 2;
                to: -2;
                duration: 150;
            }
            RotationAnimation {
                target: deleteicon;
                from: 2;
                to: -5;
                duration: 120;
            }
        }
    }

    Binding { target: selecticon; property: "selected"; value: root.selected }

    Rectangle {
        property bool selected: false

        id: selecticon;
        width: selecticonWidth;
        height: selecticonWidth;
        radius: selecticon.width/2;
        color: "transparent";
        scale: 0;
        anchors.left: appicon.right;
        anchors.leftMargin: (-selecticonWidth/2);
        anchors.top: appicon.top;
        anchors.topMargin: (-selecticonWidth/5);
        Image {
            id: selecticonImage;
            width: parent.width;
            height: parent.height;
            source: selecticon.selected ? "qrc:/ui/images/ukui_choice_click" :
                             "qrc:/ui/images/ukui_choice_default";
            anchors.centerIn: parent;
        }
        MouseArea {
            anchors.fill: parent;
            onClicked: {
                selecticon.selected = !selecticon.selected;
                modelManager.setItemSelectState(appId, pageIndex, selecticon.selected);
            }
        }
    }
    ScaleAnimator {
        id: selecticonAnimation;
        target: selecticon;
        from: 0;
        to: 1;
        duration: 500;
    }

    Component.onCompleted: {
        if(iconState === LauncherItem.Selectable) {
            selecticonAnimation.start();
            deleteicon.visible = false;
        }
    }
}
