import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQml.Models 2.2
import QtQuick.Window 2.2
import QtQuick.Layouts 1.3
import QtQml 2.12
import org.ukui.tablet 1.0
import "."

Item {
    id: root
    function listViewItemCount() {
        return listView.count
    }

    property var viewMoel: null
    property bool dragEnable: true
    property int groupId: -1
    property real cellSize: Style.widthScaleFactor * 120
    property real cellContentMargin: Style.widthScaleFactor * 65
    property var pageContainer: listView
    property bool flicking: false

    signal startDrag(bool start)
    signal containerClicked()
    // 点击分组内的“添加”按钮显示添加应用的界面
    signal showAddAppWindow(int groupId, int pageIndex)
    signal focusStateChanged()

    clip: true
    ListView {
        id: listView
        anchors.fill: parent
        spacing: 100
        snapMode: ListView.SnapOneItem
        model: viewMoel
        delegate: listViewItemDelegate
        orientation: ListView.Horizontal
        highlightRangeMode: ListView.StrictlyEnforceRange
        boundsBehavior:Flickable.StopAtBounds
        highlightMoveDuration: 500
        highlightFollowsCurrentItem: true
        cacheBuffer: listView.width * listView.count
        clip: false
        focus: true
    }
    Component {
        id: listViewItemDelegate
        KGridView {
            id: gridView
            width: listView.width
            height: listView.height
            cellWidth: cellSize
            cellHeight: cellSize
            model: modelData
            delegate: gridviewItemDelegate
            add: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            addDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            move: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            moveDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            remove: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            removeDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            displaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            interactive: false
            MouseArea {
                anchors.fill: parent
                z: -1
                onClicked: {
                    //do to edit focus
                    root.focusStateChanged()

                    modelManager.setItemsInGroupState(LauncherItem.Normal, groupId)
                }
                onPressAndHold: {
                    startDrag(true)
                    modelManager.setItemsInGroupState(LauncherItem.Editable, groupId)
                }
            }
        }
    }

    Component {
        id: gridviewItemDelegate
        DropArea {
            width: modelData.columnSpan * cellSize
            height: modelData.rowSpan * cellSize
            opacity: 1

            property int visualIndex: index
            property int pageIndex: modelData.pageIndex
            property int rowSpan: modelData.columnSpan
            property int columnSpan: modelData.columnSpan
            Binding { target: itemRoot; property: "visualIndex"; value: visualIndex }
            Binding { target: itemRoot; property: "pageIndex"; value: pageIndex }
            Binding { target: itemRoot; property: "rowSpan"; value: rowSpan }
            Binding { target: itemRoot; property: "columnSpan"; value: columnSpan }

            onEntered: {
                console.log("LauncherItemView onEntered",
                            drag.source.pageIndex, pageIndex, modelData.pageIndex, drag.source.visualIndex, visualIndex, dragEnable)
                if (!dragEnable || modelData.type === LauncherItem.AddIcon || drag.source.groupId !== root.groupId) {
                    return
                }
                if (drag.source.visualIndex === undefined) {
                    return
                }
                if (drag.source.pageIndex !== pageIndex) {
                    modelManager.addPlaceholderItem(
                                visualIndex, pageIndex,
                                drag.source.rowSpan,
                                drag.source.columnSpan,
                                groupId)
                } else {
                    modelManager.moveItem(drag.source.visualIndex, visualIndex, pageIndex, groupId)
                    modelManager.removePlaceholderItem();
                }
            }

            Item {
                id: itemRoot
                width: modelData.columnSpan * cellSize
                height: modelData.rowSpan * cellSize
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                clip: false

                property int visualIndex: 0
                property int pageIndex: 0
                property int rowSpan: 1
                property int columnSpan: 1
                property int groupId: root.groupId

                Drag.active: dragMousearea.drag.active
                Drag.source: itemRoot
                Drag.hotSpot.x: itemRoot.width / 2
                Drag.hotSpot.y: itemRoot.height / 2

                states: [
                    State {
                        when: itemRoot.Drag.active
                        ParentChange {
                            target: itemRoot
                            parent: listView
                        }

                        AnchorChanges {
                            target: itemRoot
                            anchors.horizontalCenter: undefined
                            anchors.top: undefined
                        }
                    }
                ]

                MouseArea {
                    id: dragMousearea
                    anchors.fill: parent
                    hoverEnabled: false
                    pressAndHoldInterval: modelData.state === LauncherItem.Editable ? 100 : undefined
                    property bool isScrolling: false
                    drag {
                        maximumX: listView.width - itemRoot.width/2 + 10
                        minimumX: -itemRoot.width/2 - 10
                        maximumY: listView.height - itemRoot.height/2
                        minimumY: -itemRoot.height/2
                    }

                    Timer {
                        id: scrollingTimer
                        interval: 600
                        onTriggered: dragMousearea.isScrolling = false
                    }

                    onPressAndHold: {
                        if (modelData.type === LauncherItem.AddIcon) {
                            return;
                        }
                        startDrag(true)
                        modelManager.setItemsInGroupState(LauncherItem.Editable, groupId)
                        dragMousearea.Drag.active = true
                        dragMousearea.drag.target = itemRoot
                    }

                    onClicked: {
                        if (modelData.type === LauncherItem.AddIcon) {
                            showAddAppWindow(root.groupId, modelData.pageIndex)
                        } else if (modelData.type === LauncherItem.AppIcon && modelData.state === LauncherItem.Normal) {
                            appManager.runApplication(modelData.desktopFile)
                        }
                        console.log(pressAndHoldInterval)
                    }
                    onPositionChanged:  {
                        var itemPos = item.mapToItem(listView, 0, 0)
                        if ((itemPos.x % listView.width) >= (listView.width - item.width/2) &&
                                dragMousearea.Drag.active &&!isScrolling ) {
                            if (listView.currentIndex < listView.count - 1) {
                                listView.currentIndex ++
                                isScrolling = true
                                scrollingTimer.start()
                                modelManager.removePlaceholderItem()
                                // 拖动至不同页时才添加 placeholder item
                                if (drag.target.pageIndex !== listView.currentIndex) {
                                    var index = modelManager.groupPageItemCount(groupId, listView.currentIndex) > 0 ?
                                                modelManager.groupPageItemCount(groupId, listView.currentIndex) - 1 : 0
                                    modelManager.addPlaceholderItem(index, listView.currentIndex, drag.target.rowSpan,
                                                drag.target.columnSpan, groupId)
                                }
                            } else {
                                item.x = 0
                            }
                        } else if ((itemPos.x % listView.width < -item.width/2) &&
                                   dragMousearea.Drag.active && !isScrolling) {
                            if (listView.currentIndex > 0) {
                                listView.currentIndex --
                                isScrolling = true
                                scrollingTimer.start()
                                modelManager.removePlaceholderItem()
                                if (drag.target.pageIndex !== listView.currentIndex) {
                                    index = modelManager.groupPageItemCount(groupId, listView.currentIndex) > 0 ?
                                            modelManager.groupPageItemCount(groupId, listView.currentIndex) - 1 : 0
                                    modelManager.addPlaceholderItem(index, listView.currentIndex, drag.target.rowSpan,
                                                drag.target.columnSpan, groupId)
                                }
                            }
                        }
                    }
                    onPressed: {
                        if (modelData.type === LauncherItem.AddIcon) {
                            iconpressAnimation.target = item
                            iconpressAnimation.start()
                        }
                    }

                    onReleased: {
                        if (modelData.type === LauncherItem.AddIcon) {
                            iconreleseAnimation.target = item
                            iconreleseAnimation.start()
                            return
                        }
                        startDrag(false)
                        isScrolling = false
                        dragMousearea.Drag.active = false
                        dragMousearea.drag.target = null
                        modelManager.replacePlaceholderItem(itemRoot.visualIndex, itemRoot.pageIndex, groupId)
                        modelManager.updateCachedData(groupId)
                    }
                }
                AppIcon {
                    id: item
                    iconWidth: modelData.columnSpan * cellSize - cellContentMargin
                    visible: modelData.type !== LauncherItem.Placeholder
                    disableDelete: !modelManager.groupEditable(root.groupId)
                    opacity: {
                        if (modelData.type === LauncherItem.AppIcon) {
                            if (listView.moving) return 1
                            if (dragMousearea.pressed) return 0.5
                        }
                        return 1
                    }
                    icon: {
                        switch (modelData.type) {
                        case LauncherItem.AppIcon: return "image://icons/" + modelData.icon
                        case LauncherItem.AddIcon:
                            if (dragMousearea.pressed) {
                                return "qrc:/ui/images/list-add-click.svg";
                            } else {
                                return "qrc:/ui/images/list-add-default.svg";
                            }
                        default: return ""
                        }
                    }
                    iconState: modelData.state
                    anchors {
                        horizontalCenter: parent.horizontalCenter;
                        verticalCenter: parent.verticalCenter
                    }
                    onDeleteIcon: {
                        modelManager.removeItem(itemRoot.visualIndex, itemRoot.pageIndex, groupId)
                    }
                    onIconStateChanged: {
                        if (modelData.type === LauncherItem.AddIcon) {
                            if (iconState === LauncherItem.Normal) {
                                item.visible = true
                                itemName.visible = true
                                dragMousearea.enabled = true
                            } else {
                                item.visible = false
                                itemName.visible = false
                                dragMousearea.enabled = false
                            }
                        }
                    }
                    Component.onCompleted: console.log("--------------",modelData.type)
                }
                Text {
                    id: itemName
                    width: (item.width + 40) * Style.widthScaleFactor
                    anchors { top: item.bottom; topMargin: 12;
                        horizontalCenter: item.horizontalCenter }
                    horizontalAlignment: Text.AlignHCenter
                    color: "black"
                    text: model.modelData.name
                    elide: Text.ElideRight
                    smooth: true
                    visible: modelData.type !== LauncherItem.Placeholder
                }
                Component.onCompleted: {
                    if (viewMoel === null) {
                        console.warn("AppContainer: The model is null, please assign a model!")
                    }
                }
            }
        }
    }
    ScaleAnimator {
        id: iconpressAnimation
        from: 1
        to: 0.8
        duration: 150
    }
    ScaleAnimator {
        id: iconreleseAnimation
        from: 0.8
        to: 1
        duration: 150
    }

    onContainerClicked: {
        modelManager.setItemsInGroupState(LauncherItem.Normal, groupId)
    }
}






