import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Window 2.0
import QtQml.Models 2.2
import QtQuick.Window 2.2
import QtQuick.Layouts 1.3
import QtQml 2.12
import org.ukui.tablet 1.0
import "."

Item {
    id: root
    property var viewMoel: null
    property bool dragEnable: true
    property real cellSize: Style.widthScaleFactor * 130
    property real gridCellHeight: Style.heightScaleFactor * 130
    property real cellContentMargin: Style.widthScaleFactor * 20

    signal startDrag(bool started)
    signal showAddAppWindow(int groupId, int pageIndex)
    signal titleFocusChanged()

    clip: true

    ListView {
        id: listView
        anchors.fill: parent
        spacing: Style.widthScaleFactor * 40
        snapMode: ListView.SnapOneItem
        model: viewMoel
        delegate: listViewItemDelegate
        orientation: ListView.Horizontal
        highlightRangeMode: ListView.StrictlyEnforceRange
        boundsBehavior:Flickable.StopAtBounds
        highlightFollowsCurrentItem: true
        highlightMoveDuration: 500
        cacheBuffer: listView.width * listView.count
        clip: false
        focus: true
        interactive: false
    }
    Component {
        id: listViewItemDelegate
        KGridView {
            id: gridView
            width: listView.width
            height: listView.height
            cellWidth: cellSize
            cellHeight: gridCellHeight
            model: modelData
            interactive: false
            delegate: gridviewItemDelegate
            add: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            addDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            move: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            moveDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            remove: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            removeDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            displaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
        }
    }

    Component {
        id: gridviewItemDelegate
        DropArea {
            id: delegateContent
            width: modelData.columnSpan * cellSize
            height: modelData.rowSpan * gridCellHeight
            opacity: 1

            property int visualIndex: index
            property int pageIndex: modelData.pageIndex
            property int rowSpan: modelData.columnSpan
            property int columnSpan: modelData.columnSpan
            Binding { target: itemRoot; property: "visualIndex"; value: visualIndex }
            Binding { target: itemRoot; property: "pageIndex"; value: pageIndex }
            Binding { target: itemRoot; property: "rowSpan"; value: rowSpan }
            Binding { target: itemRoot; property: "columnSpan"; value: columnSpan }

            Component.onCompleted: {
                itemRoot.visualIndex = index
            }

            onEntered: {
                console.log("LauncherView onEntered", drag.source.pageIndex, pageIndex, modelData.pageIndex, drag.source.visualIndex, visualIndex)
                if (!dragEnable) {
                    return
                }
                console.log("LauncherView onEntered")
                if (drag.source.visualIndex === undefined) {
                    return
                }
                if (drag.source.pageIndex !== pageIndex) {
                    modelManager.addPlaceholderItem(visualIndex, pageIndex, drag.source.rowSpan, drag.source.columnSpan)
                } else {
                    modelManager.moveItem(drag.source.visualIndex, visualIndex, pageIndex)
                    modelManager.removePlaceholderItem();
                }
            }
            onDropped: {
                console.log("dropped.")
            }
            Item {
                id: itemRoot
                width: modelData.columnSpan * cellSize
                height: modelData.rowSpan * gridCellHeight
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                clip: false

                // 方便在拖动时获取数据
                property int visualIndex: 0
                property int pageIndex: 0
                property int rowSpan: 1
                property int columnSpan: 1

                Drag.active: dragMousearea.drag.active && dragEnable
                Drag.source: itemRoot
                Drag.hotSpot.x: itemRoot.width / 2
                Drag.hotSpot.y: 0//itemRoot.height / 2

                states: [
                    State {
                        when: itemRoot.Drag.active
                        ParentChange {
                            target: itemRoot
                            parent: listView
                        }

                        AnchorChanges {
                            target: itemRoot
                            anchors.horizontalCenter: undefined
                            anchors.top: undefined
                        }
                    }
                ]

                MouseArea {
                    id: dragMousearea
                    anchors.fill: parent
                    hoverEnabled: true
                    Drag.active: false
                    drag.target: undefined
                    property bool isScrolling: false

                    Timer {
                        id: scrollingTimer
                        interval: 1000
                        onTriggered: dragMousearea.isScrolling = false
                    }

                    onPressAndHold: {
                        console.log("press and hold")
                        if (itemRoot.pageIndex === 0) {
                            return
                        }
                        dragMousearea.Drag.active = true
                        dragMousearea.drag.target = itemRoot
                        startDrag(true)
                    }
                    onClicked: {
                        console.log("LauncherView clicked")
                        root.titleFocusChanged()
                    }
                    onPositionChanged:  {
                        var itemPos = item.mapToItem(listView, 0, 0)
                        if ((itemPos.x % listView.width) >= (listView.width - 100) &&
                                dragMousearea.Drag.active &&!isScrolling ) {
                            if (listView.currentIndex < listView.count - 1) {
                                listView.currentIndex ++
                                isScrolling = true
                                scrollingTimer.start()
                                modelManager.addPlaceholderItem(modelManager.launcerPageItemCount(listView.currentIndex),
                                                                listView.currentIndex,
                                                                drag.target.rowSpan,
                                                                drag.target.columnSpan,
                                                                groupId)
                            }
                        } else if ((itemPos.x % listView.width < 100) &&
                                   dragMousearea.Drag.active && !isScrolling) {
                            if (listView.currentIndex > 0) {
                                listView.currentIndex --
                                isScrolling = true
                                scrollingTimer.start()
                                modelManager.addPlaceholderItem(modelManager.launcerPageItemCount(listView.currentIndex),
                                                                listView.currentIndex,
                                                                drag.target.rowSpan,
                                                                drag.target.columnSpan,
                                                                groupId)
                            }
                        }
                    }
                    onReleased: {
                        isScrolling = false
                        dragMousearea.Drag.active = false
                        dragMousearea.drag.target = null
                        modelManager.replacePlaceholderItem(visualIndex, pageIndex)
                        startDrag(false)
                    }
                }

                Rectangle {
                    id: item
                    width: modelData.columnSpan * cellSize - Style.widthScaleFactor * 40
                    height: modelData.rowSpan * gridCellHeight - Style.heightScaleFactor * 40
                    color: "transparent"
                    clip: true
                    radius: 5
                    anchors {
                        horizontalCenter: parent.horizontalCenter;
                        verticalCenter: parent.verticalCenter
                    }
                    Image {
                        id: appIcon
                        anchors.fill: parent
                        smooth: true
                        visible: modelData.type === LauncherItem.AppIcon
                        source: modelData.type === LauncherItem.AppIcon ? "file://" + modelData.icon : ""
                    }

                    Component.onCompleted: {
                        if (modelData.type === LauncherItem.Container) {
                            itemName.visible = false
                            var component = Qt.createComponent("AppContainer.qml");
                            if (component.status === Component.Ready) {
                                var titleEditable = modelData.titleEditable

                                var container = component.createObject(item, { width: item.width,
                                                                           height: item.height,
                                                                           model: modelManager.groupModel(modelData.id),
                                                                           containerTitle: modelData.name,
                                                                           groupId: modelData.id,
                                                                           titleEditable: titleEditable,
                                                                           blurWindow: root.blurWindow, rootItem: root.rootItem });

                                container.width = Qt.binding(function() {
                                    return item.width
                                })
                                container.height = Qt.binding(function() {
                                    return item.height
                                })

                                root.startDrag.connect(container.enableItemDrag)
                                root.titleFocusChanged.connect(container.editFocusChange)
                                container.itemStartDrag.connect(function(start) {
                                    dragEnable = !start
                                    console.log("item in container start dragging...", start)
                                })
                                container.showAddAppWindow.connect(root.showAddAppWindow)
                            } else {
                                console.log("error string: ", component.errorString() )
                            }
                        } else if (modelData.type === LauncherItem.Group) {
                            itemName.visible = false
                            var groupcomponent = Qt.createComponent("AppContainer.qml");
                            if (groupcomponent.status === Component.Ready) {
                                var group = groupcomponent.createObject(item, { enable: false, scale: Style.widthScaleFactor * 0.2, transformOrigin: Item.TopLeft,
                                                                            model: modelManager.groupModel(modelData.id) });
                                group.enableItemDrag(true)
                            }
                        } else if (modelData.type === LauncherItem.Widget) {
                            var loginComponent = Qt.createComponent("LoginComponent.qml");
                            if (loginComponent.status === Component.Ready) {
                                loginComponent.createObject(item, { width: item.width, height: item.height })
                            }
                        }
                    }
                }
                Text {
                    id: itemName
                    width: item.width
                    anchors.top: item.bottom
                    anchors.horizontalCenter: item.horizontalCenter
                    color: "black"
                    text: model.modelData.name
                    elide: Text.ElideRight
                    smooth: true
                    visible: modelData.type === LauncherItem.AppIcon
                }
                Component.onCompleted: {
                    if (viewMoel === null) {
                        console.warn("AppContainer: The model is null, please assign a model!")
                    }
                }
            }
        }
    }
}






