import QtQuick 2.12
import QtQuick.Layouts 1.12
import "."

Rectangle {
    id: root
    property alias title: titleInput.text
    property bool readOnly: true
    property bool isEditing: false
    property var txtwidth: root.width
    property var titleTxt: titleInput.text

    signal editVisible()
    signal editInVisible()
    width:  (titleInput.contentWidth < 1) ? (70 * Style.widthScaleFactor)  :
                                         (titleInput.contentWidth + 70) * Style.widthScaleFactor

    height: 30 * Style.heightScaleFactor
    radius: 10
    color: "white"
    TextInput {
        id: titleInput
        anchors.fill: parent
        horizontalAlignment: TextInput.AlignHCenter
        verticalAlignment: TextInput.AlignVCenter

        maximumLength: 8
        font.pointSize: Style.widthScaleFactor * 14
        clip: true
        readOnly: true
        focus: false
        text: title
        rightPadding: 10 * Style.widthScaleFactor

        MouseArea{
            anchors.fill: parent
            onClicked: {
                if (titleEditable === false){
                    container.focusChange()
                    container.clicked()
                }
            }
            onPressAndHold: {
                //不同Container是否可编辑状态的判断
                //titleEditable为true是为可以编辑的状态
                if (titleEditable === true && image.state === "Normal"){
                    if (image.state === "Normal") {
                        titleInput.readOnly = false
                        titleInput.focus = true
                        titleInput.horizontalAlignment = TextInput.AlignLeft
                        titleInput.rightPadding= Qt.binding(function() {
                            return image.width * Style.widthScaleFactor
                        })
                        image.state = "Editable"
                        if(image.state === "Editable"){
                            root.width = Qt.binding(function() {
                                return 400 * Style.widthScaleFactor
                            })
                            isEditing = true
                            root.editVisible()
                            title = titleInput.text
                            if(title == ""){
                                title = root.titleTxt
                            }
                            else{
                                 root.titleTxt = title
                            }
                        }
                    }
                    else {
                        titleInput.clear()
                        titleInput.Layout.preferredWidth = Qt.binding(function() {
                            return 400 * Style.widthScaleFactor
                        })
                    }
                }
            }
        }

        Image {
            id: image
            visible: !readOnly
            anchors { right: titleInput.right; rightMargin:  5 }
            anchors.verticalCenter: parent.verticalCenter
            sourceSize { width: 16 * Style.widthScaleFactor; height: 16 * Style.heightScaleFactor; }
            state: "Normal"
            states: [
                State {
                    name: "Normal"
                    PropertyChanges {
                        target: image
                        source: "qrc:/ui/images/ukui_teaching_default.svg"
                    }
                },
                State {
                    name: "Editable"
                    PropertyChanges {
                        target: image
                        source: "qrc:/ui/images/ukui_exit_symbolic.svg"
                    }
                }
            ]
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (image.state === "Normal") {
                        titleInput.readOnly = false
                        titleInput.focus = true
                        titleInput.horizontalAlignment = TextInput.AlignLeft
                        titleInput.rightPadding= Qt.binding(function() {
                            return image.width  * Style.widthScaleFactor
                        })
                        image.state = "Editable"
                        if(image.state === "Editable"){
                            root.width = Qt.binding(function() {
                                return 400 * Style.widthScaleFactor
                            })
                            isEditing = true
                            root.editVisible()
                            title = titleInput.text
                            if(title == ""){
                                title = root.titleTxt
                            }
                            else{
                                 root.titleTxt = title
                            }
                        }
                    }
                    else {
                        titleInput.clear()
                        root.width = Qt.binding(function() {
                            return 400 * Style.widthScaleFactor
                        })
                    }
                }
            }
        }

        onEditingFinished: {
            readOnly = true
            focus = false
            titleInput.horizontalAlignment = TextInput.AlignHCenter
            titleInput.rightPadding= Qt.binding(function() {
                return image.width * Style.widthScaleFactor
            })
            title = titleInput.text
            image.state = "Normal"
            isEditing = false
            root.width = Qt.binding(function() {
                return titleInput.contentWidth < 1 ?
                            (70 * Style.widthScaleFactor) : (titleInput.contentWidth + 70) * Style.widthScaleFactor
            })
            root.editInVisible()
            if(text == ""){
                text = root.titleTxt
                root.width = Qt.binding(function() {
                    return titleInput.contentWidth < 1 ?
                                (70 * Style.widthScaleFactor)  : (titleInput.contentWidth + 15) * Style.widthScaleFactor
                })
                root.editInVisible()
            }
            else{
                 root.titleTxt = text
            }
        }
    }

    function focusHandle(){
        container.focus = true
    }
}
