#ifndef MISC_H
#define MISC_H

#include <QVariant>
#include <QDBusPendingReply>
#include <QDBusInterface>
#include <QThread>
#include <QApplication>

class Misc
{
public:
    Misc();

    template <typename T>
    static QDBusPendingReply<T> DbusAsyncCall(const QString &service,
                                              const QString &path,
                                              const QString &interface,
                                              const QDBusConnection &connection,
                                              const QString &method)
    {
        QDBusInterface dbusInterface(service, path, interface, connection);
        QDBusPendingReply<T> reply = dbusInterface.asyncCall(method);
        while (!reply.isFinished()) {
            qApp->processEvents();
            QThread::msleep(1);
        }
        return reply;
    }

    template <typename T>
    static QDBusPendingReply<T> DbusAsyncCallWithArgs(const QString &service,
                                                      const QString &path,
                                                      const QString &interface,
                                                      const QDBusConnection &connection,
                                                      const QString &method,
                                                      const QList<QVariant> &argList)
    {
        QDBusInterface dbusInterface(service, path, interface, connection);
        QDBusPendingReply<T> reply = dbusInterface.asyncCallWithArgumentList(method, argList);
        while (!reply.isFinished()) {
            qApp->processEvents();
            QThread::msleep(1);
        }
        return reply;
    }
};

#endif // MISC_H
